@@ -59,6 +59,8 @@ private slots:
     void volume_data(){generate_audiofile_testrows();}
     void volume();
 
+    void stop_finishesPushMode_whenInvokedUponReadyReadSignal();
+
 private:
     using FilePtr = QSharedPointer<QFile>;
 
@@ -292,7 +294,7 @@ void tst_QAudioSource::stopWhileStopped()
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
     QVERIFY2((audioInput.error() == QAudio::NoError), "error() was not set to QAudio::NoError before start()");
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
     audioInput.stop();
 
     // Check that no state transition occurred
@@ -312,7 +314,7 @@ void tst_QAudioSource::suspendWhileStopped()
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
     QVERIFY2((audioInput.error() == QAudio::NoError), "error() was not set to QAudio::NoError before start()");
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
     audioInput.suspend();
 
     // Check that no state transition occurred
@@ -332,7 +334,7 @@ void tst_QAudioSource::resumeWhileStopped()
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
     QVERIFY2((audioInput.error() == QAudio::NoError), "error() was not set to QAudio::NoError before start()");
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
     audioInput.resume();
 
     // Check that no state transition occurred
@@ -347,7 +349,7 @@ void tst_QAudioSource::pull()
 
     QAudioSource audioInput(audioFormat, this);
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
     // Check that we are in the default state before calling start
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -416,7 +418,7 @@ void tst_QAudioSource::pullSuspendResume()
 
     QAudioSource audioInput(audioFormat, this);
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
     // Check that we are in the default state before calling start
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -518,7 +520,7 @@ void tst_QAudioSource::push()
 
     QAudioSource audioInput(audioFormat, this);
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
     // Check that we are in the default state before calling start
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -609,7 +611,7 @@ void tst_QAudioSource::pushSuspendResume()
 
     audioInput.setBufferSize(audioFormat.bytesForDuration(100000));
 
-    QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+    QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
     // Check that we are in the default state before calling start
     QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -735,7 +737,7 @@ void tst_QAudioSource::reset()
     {
         QAudioSource audioInput(audioFormat, this);
 
-        QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+        QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
         // Check that we are in the default state before calling start
         QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -765,7 +767,7 @@ void tst_QAudioSource::reset()
         QBuffer buffer;
         buffer.open(QIODevice::WriteOnly);
 
-        QSignalSpy stateSignal(&audioInput, SIGNAL(stateChanged(QAudio::State)));
+        QSignalSpy stateSignal(&audioInput, &QAudioSource::stateChanged);
 
         // Check that we are in the default state before calling start
         QVERIFY2((audioInput.state() == QAudio::StoppedState), "state() was not set to StoppedState before start()");
@@ -812,6 +814,37 @@ void tst_QAudioSource::volume()
     audioInput.setVolume(volume);
 }
 
+void tst_QAudioSource::stop_finishesPushMode_whenInvokedUponReadyReadSignal()
+{
+    const auto defaultAudioInputDevice = QMediaDevices::defaultAudioInput();
+
+    QAudioFormat audioFormat;
+    audioFormat.setSampleFormat(QAudioFormat::Int16);
+    audioFormat.setSampleRate(qBound(defaultAudioInputDevice.minimumSampleRate(), 48000,
+                                     defaultAudioInputDevice.maximumSampleRate()));
+    audioFormat.setChannelCount(qBound(defaultAudioInputDevice.minimumChannelCount(), 2,
+                                       defaultAudioInputDevice.maximumChannelCount()));
+
+    const auto isFormatSupported = defaultAudioInputDevice.isFormatSupported(audioFormat);
+    QCOMPARE(isFormatSupported, true);
+
+    QAudioSource audioInput(audioFormat, this);
+
+    const auto audioInputDevice = audioInput.start();
+
+    auto isReadyReadReceived = false;
+    connect(audioInputDevice, &QIODevice::readyRead, this, [&]() {
+        audioInput.stop();
+        isReadyReadReceived = true;
+    });
+
+    const auto awaitedValue = QTest::qWaitFor([&] { return isReadyReadReceived; });
+    QVERIFY2(awaitedValue, "didn't receive readyRead signal");
+
+    QVERIFY2((audioInput.state() == QAudio::StoppedState),
+             "didn't transitions to StoppedState after close()");
+}
+
 QTEST_MAIN(tst_QAudioSource)
 
 #include "tst_qaudiosource.moc"
